@@ -304,8 +304,8 @@ error:
 
 /* {{{ Compression helpers */
 #ifdef USE_ZLIB
-static int _PylibMC_Deflate(char *value, size_t value_len,
-                    char **result, size_t *result_len,
+static int _PylibMC_Deflate(char *value, Py_ssize_t value_len,
+                    char **result, Py_ssize_t *result_len,
                     int compress_level) {
     /* FIXME Failures are entirely silent. */
     int rc;
@@ -373,8 +373,8 @@ error:
     return 0;
 }
 
-static int _PylibMC_Inflate(char *value, size_t size,
-                            char** result, size_t* result_size,
+static int _PylibMC_Inflate(char *value, Py_ssize_t size,
+                            char** result, Py_ssize_t* result_size,
                             char** failure_reason) {
 
     /*
@@ -542,7 +542,7 @@ static void _PylibMC_cleanup_str_key_mapping(PyObject *key_str_map) {
 /* }}} */
 
 static PyObject *_PylibMC_parse_memcached_value(PylibMC_Client *self,
-        char *value, size_t size, uint32_t flags) {
+        char *value, Py_ssize_t size, uint32_t flags) {
     PyObject *retval = NULL;
 
 #if USE_ZLIB
@@ -552,7 +552,7 @@ static PyObject *_PylibMC_parse_memcached_value(PylibMC_Client *self,
     if (flags & PYLIBMC_FLAG_ZLIB) {
         int rc;
         char* inflated_buf = NULL;
-        size_t inflated_size = 0;
+        Py_ssize_t inflated_size = 0;
         char* failure_reason = NULL;
 
         if(size >= ZLIB_GIL_RELEASE) {
@@ -618,7 +618,7 @@ static PyObject *_PylibMC_parse_memcached_value(PylibMC_Client *self,
 }
 
 /** Helper because PyLong_FromString requires a null-terminated string. */
-static PyObject *_PyLong_FromStringAndSize(char *value, size_t size, char **pend, int base) {
+static PyObject *_PyLong_FromStringAndSize(char *value, Py_ssize_t size, char **pend, int base) {
     PyObject *retval;
     char *tmp;
     if ((tmp = malloc(size+1)) == NULL) {
@@ -637,7 +637,7 @@ static PyObject *_PyLong_FromStringAndSize(char *value, size_t size, char **pend
   the value to be deserialized is a byte array `value_str` of length
   `value_size`.
   */
-static PyObject *_PylibMC_deserialize_native(PylibMC_Client *self, PyObject *value, char *value_str, size_t value_size, uint32_t flags) {
+static PyObject *_PylibMC_deserialize_native(PylibMC_Client *self, PyObject *value, char *value_str, Py_ssize_t value_size, uint32_t flags) {
     assert(value || value_str);
     PyObject *retval = NULL;
 
@@ -707,47 +707,54 @@ static int _PylibMC_cache_miss_simulated(PyObject *r) {
     return 0;
 }
 
-static PyObject *PylibMC_Client_get(PylibMC_Client *self, PyObject *arg) {
+static PyObject *PylibMC_Client_get(PylibMC_Client *self, PyObject *args) {
     char *mc_val;
     size_t val_size;
     uint32_t flags;
     memcached_return error;
+    PyObject *key;
+    /* if a default argument was in fact passed, it's still a borrowed reference
+       at this point, so borrow a reference to Py_None as well for parity. */
+    PyObject *default_value = Py_None;
 
-    if (!_key_normalized_obj(&arg)) {
+    if (!PyArg_UnpackTuple(args, "get", 1, 2, &key, &default_value)) {
         return NULL;
-    } else if (!PySequence_Length(arg) ) {
-        Py_RETURN_NONE;
+    }
+
+    if (!_key_normalized_obj(&key)) {
+        return NULL;
+    } else if (!PySequence_Length(key) ) {
+        Py_INCREF(default_value);
+        return default_value;
     }
 
     Py_BEGIN_ALLOW_THREADS;
     mc_val = memcached_get(self->mc,
-            PyBytes_AS_STRING(arg), PyBytes_GET_SIZE(arg),
+            PyBytes_AS_STRING(key), PyBytes_GET_SIZE(key),
             &val_size, &flags, &error);
     Py_END_ALLOW_THREADS;
 
-    Py_DECREF(arg);
+    Py_DECREF(key);
 
     if (mc_val != NULL) {
         PyObject *r = _PylibMC_parse_memcached_value(self, mc_val, val_size, flags);
         free(mc_val);
         if (_PylibMC_cache_miss_simulated(r)) {
-            /* Since python-memcache returns None when the key doesn't exist,
-             * so shall we. */
-            Py_RETURN_NONE;
+            Py_INCREF(default_value);
+            return default_value;
         }
         return r;
     } else if (error == MEMCACHED_SUCCESS) {
         /* This happens for empty values, and so we fake an empty string. */
         return PyBytes_FromStringAndSize("", 0);
     } else if (error == MEMCACHED_NOTFOUND) {
-        /* Since python-memcache returns None when the key doesn't exist,
-         * so shall we. */
-        Py_RETURN_NONE;
+        Py_INCREF(default_value);
+        return default_value;
     }
 
     return PylibMC_ErrFromMemcachedWithKey(self, "memcached_get", error,
-                                           PyBytes_AS_STRING(arg),
-                                           PyBytes_GET_SIZE(arg));
+                                           PyBytes_AS_STRING(key),
+                                           PyBytes_GET_SIZE(key));
 }
 
 static PyObject *PylibMC_Client_gets(PylibMC_Client *self, PyObject *arg) {
@@ -829,7 +836,7 @@ static PyObject *PylibMC_Client_hash(PylibMC_Client *self, PyObject *args, PyObj
         return NULL;
     }
 
-    h = memcached_generate_hash(self->mc, key, (size_t)key_len);
+    h = memcached_generate_hash(self->mc, key, (Py_ssize_t)key_len);
 
     return PyLong_FromLong((long)h);
 }
@@ -922,11 +929,11 @@ static PyObject *_PylibMC_RunSetCommandMulti(PylibMC_Client *self,
     unsigned int min_compress = 0;
     int compress_level = -1;
     PyObject *failed = NULL;
-    size_t idx = 0;
+    Py_ssize_t idx = 0;
     PyObject *curr_key, *curr_value;
     PyObject *key_str_map = NULL;
     Py_ssize_t i;
-    size_t nkeys;
+    Py_ssize_t nkeys;
     pylibmc_mset* serialized = NULL;
     bool allsuccess;
 
@@ -955,7 +962,7 @@ static PyObject *_PylibMC_RunSetCommandMulti(PylibMC_Client *self,
     }
 #endif
 
-    nkeys = (size_t)PyDict_Size(keys);
+    nkeys = (Py_ssize_t)PyDict_Size(keys);
 
     key_str_map = _PylibMC_map_str_keys(keys, NULL, NULL);
     if (key_str_map == NULL) {
@@ -1294,8 +1301,8 @@ static PyObject *PylibMC_Client_serialize(PylibMC_Client *self, PyObject *value_
 /* {{{ Set commands (set, replace, add, prepend, append) */
 static bool _PylibMC_RunSetCommand(PylibMC_Client* self,
                                    _PylibMC_SetCommand f, char *fname,
-                                   pylibmc_mset* msets, size_t nkeys,
-                                   size_t min_compress,
+                                   pylibmc_mset* msets, Py_ssize_t nkeys,
+                                   Py_ssize_t min_compress,
                                    int compress_level) {
     memcached_st *mc = self->mc;
     memcached_return rc = MEMCACHED_SUCCESS;
@@ -1309,12 +1316,12 @@ static bool _PylibMC_RunSetCommand(PylibMC_Client* self,
         pylibmc_mset *mset = &msets[i];
 
         char *value = mset->value;
-        size_t value_len = (size_t)mset->value_len;
+        Py_ssize_t value_len = (Py_ssize_t)mset->value_len;
         uint32_t flags = mset->flags;
 
 #ifdef USE_ZLIB
         char *compressed_value = NULL;
-        size_t compressed_len = 0;
+        Py_ssize_t compressed_len = 0;
 
         if (compress_level && min_compress && value_len >= min_compress) {
             _PylibMC_Deflate(value, value_len,
@@ -1501,11 +1508,6 @@ static PyObject *_PylibMC_IncrSingle(PylibMC_Client *self,
         return NULL;
     }
 
-    if ((unsigned int)delta != delta) {
-        PyErr_Format(PyExc_OverflowError, "%d", delta);
-        return NULL;
-    }
-
     incr.key = key;
     incr.key_len = key_len;
     incr.incr_func = incr_func;
@@ -1535,7 +1537,7 @@ static PyObject *_PylibMC_IncrMulti(PylibMC_Client *self,
     PyObject *retval = NULL;
     PyObject *iterator = NULL;
     unsigned int delta = 1;
-    size_t nkeys = 0, i = 0;
+    Py_ssize_t nkeys = 0, i = 0;
     pylibmc_incr *incrs;
 
     static char *kws[] = { "keys", "key_prefix", "delta", NULL };
@@ -1545,7 +1547,7 @@ static PyObject *_PylibMC_IncrMulti(PylibMC_Client *self,
                                      &key_prefix_len, &delta))
         return NULL;
 
-    nkeys = (size_t)PySequence_Size(keys);
+    nkeys = (Py_ssize_t)PySequence_Size(keys);
     if (nkeys == -1)
         return NULL;
 
@@ -1638,10 +1640,10 @@ static PyObject *PylibMC_Client_incr_multi(PylibMC_Client *self, PyObject *args,
 }
 
 static bool _PylibMC_IncrDecr(PylibMC_Client *self,
-                              pylibmc_incr *incrs, size_t nkeys) {
+                              pylibmc_incr *incrs, Py_ssize_t nkeys) {
     memcached_return rc = MEMCACHED_SUCCESS;
     _PylibMC_IncrCommand f = NULL;
-    size_t i, notfound = 0, errors = 0;
+    Py_ssize_t i, notfound = 0, errors = 0;
 
     Py_BEGIN_ALLOW_THREADS;
     for (i = 0; i < nkeys; i++) {
@@ -1757,7 +1759,7 @@ static PyObject *PylibMC_Client_get_multi(
     PyObject *temp_key_obj;
     size_t *key_lens;
     Py_ssize_t nkeys = 0, orig_nkeys = 0;
-    size_t nresults = 0;
+    Py_ssize_t nresults = 0;
     memcached_return rc;
     pylibmc_mget_req req;
 
@@ -1773,8 +1775,8 @@ static PyObject *PylibMC_Client_get_multi(
     /* Populate keys and key_lens. */
     keys = PyMem_New(char *, orig_nkeys);
     key_lens = PyMem_New(size_t, (size_t) orig_nkeys);
-    key_objs = PyMem_New(PyObject *, (size_t) orig_nkeys);
-    orig_key_objs = PyMem_New(PyObject *, (size_t) orig_nkeys);
+    key_objs = PyMem_New(PyObject *, (Py_ssize_t) orig_nkeys);
+    orig_key_objs = PyMem_New(PyObject *, (Py_ssize_t) orig_nkeys);
     if (!keys || !key_lens || !key_objs || !orig_key_objs) {
         PyErr_NoMemory();
         goto memory_cleanup;
@@ -1795,7 +1797,7 @@ static PyObject *PylibMC_Client_get_multi(
         PyObject *ckey = orig_key_objs[i];
         char *key;
         Py_ssize_t key_len;
-        size_t final_key_len;
+        Py_ssize_t final_key_len;
         PyObject *rkey;
 
         if (PyErr_Occurred() || !_key_normalized_obj(&ckey)) {
@@ -1807,7 +1809,7 @@ static PyObject *PylibMC_Client_get_multi(
 
         PyBytes_AsStringAndSize(ckey, &key, &key_len);
 
-        final_key_len = (size_t)(key_len + prefix_len);
+        final_key_len = (Py_ssize_t)(key_len + prefix_len);
 
         /* Skip empty keys */
         if (!final_key_len) {
@@ -2067,10 +2069,8 @@ static PyObject *PylibMC_Client_delete_multi(PylibMC_Client *self,
     const char *prefix_raw = NULL;
     Py_ssize_t prefix_len;
     PyObject *prefix = NULL;
-    PyObject *time = NULL;
     PyObject *delete;
     PyObject *keys;
-    PyObject *call_args;
     PyObject *retval;
 
     static char *kws[] = { "keys", "key_prefix", NULL };
@@ -2080,48 +2080,25 @@ static PyObject *PylibMC_Client_delete_multi(PylibMC_Client *self,
         return NULL;
 
     /**
-     * Because of how DoMulti works, we have to prohibit the use of mappings
-     * here. Otherwise, the values of the mapping will be the second argument
-     * to the delete function; the time argument will then become a third
-     * argument, which delete doesn't take.
-     *
-     * So a mapping to DoMulti would produce calls like:
-     *   DoMulti({"a": 1, "b": 2}, time=3)
-     *      delete("a", 1, 3)
-     *      delete("b", 2, 3)
-     */
-#if PY_MAJOR_VERSION >= 3
-    /*
-     * This isn't optimal, but PyMapping_Check returns 1 for
-     * tuples, lists and other sequences in Python 3. According to
-     * http://bugs.python.org/issue5945 PyMapping_Check has never
-     * been particularly reliable, so hopefully it's enough to
-     * check for dict objects (and subclasses) specifically.
+     * Prohibit use of mappings, otherwise DoMulti interprets the values of
+     * such mappings as the 2nd argument to the delete function, e.g.
+     * DoMulti({"a": 1, "b": 2}) calls delete("a", 1) and delete("b", 2), which
+     * is nonsense.
      */
     if (PyDict_Check(keys)) {
-#else
-    if (PyMapping_Check(keys)) {
-#endif
         PyErr_SetString(PyExc_TypeError,
             "keys must be a sequence, not a mapping");
         return NULL;
     }
 
-    if (prefix_raw != NULL) {
+    if (prefix_raw != NULL)
         prefix = PyBytes_FromStringAndSize(prefix_raw, prefix_len);
-    }
 
     if ((delete = PyObject_GetAttrString((PyObject *)self, "delete")) == NULL)
         return NULL;
 
-    if (time == NULL) {
-        retval = _PylibMC_DoMulti(keys, delete, prefix, NULL);
-    } else {
-        if ((call_args = PyTuple_Pack(1, time)) == NULL)
-            goto error;
-        retval = _PylibMC_DoMulti(keys, delete, prefix, call_args);
-        Py_DECREF(call_args);
-    }
+    retval = _PylibMC_DoMulti(keys, delete, prefix, NULL);
+
     Py_DECREF(delete);
     Py_XDECREF(prefix);
 
@@ -2135,12 +2112,9 @@ static PyObject *PylibMC_Client_delete_multi(PylibMC_Client *self,
         Py_DECREF(retval);
         retval = Py_False;
     }
-    Py_INCREF(retval);
 
+    Py_INCREF(retval);
     return retval;
-error:
-    Py_XDECREF(delete);
-    return NULL;
 }
 
 static PyObject *PylibMC_Client_get_behaviors(PylibMC_Client *self) {
@@ -2438,10 +2412,18 @@ static void _set_error(memcached_st *mc, memcached_return error, char *lead) {
                      lead, strerror(errno));
     } else if (error == MEMCACHED_SUCCESS) {
         PyErr_Format(PyExc_RuntimeError, "error == MEMCACHED_SUCCESS");
+#if LIBMEMCACHED_VERSION_HEX >= 0x01000002
+    } else if (error == MEMCACHED_E2BIG) {
+        PyErr_SetNone(_exc_by_rc(error));
+#endif
     } else {
         PyObject *exc = _exc_by_rc(error);
 #if LIBMEMCACHED_VERSION_HEX >= 0x00049000
-        PyErr_Format(exc, "%s: %.200s", lead, memcached_last_error_message(mc));
+        if (memcached_last_error(mc) != MEMCACHED_SUCCESS) {
+            PyErr_Format(exc, "%s: %.200s", lead, memcached_last_error_message(mc));
+        } else {
+            PyErr_SetString(exc, lead);
+        }
 #else
         PyErr_Format(exc, "%s: %.200s", lead, memcached_strerror(mc, error));
 #endif
@@ -2489,7 +2471,7 @@ static PyObject *_PylibMC_GetPickles(const char *attname) {
     return pickle_attr;
 }
 
-static PyObject *_PylibMC_Unpickle(const char *buff, size_t size) {
+static PyObject *_PylibMC_Unpickle(const char *buff, Py_ssize_t size) {
 #if PY_MAJOR_VERSION >= 3
         return PyObject_CallFunction(_PylibMC_pickle_loads, "y#", buff, size);
 #else
@@ -2502,7 +2484,7 @@ static PyObject *_PylibMC_Unpickle_Bytes(PyObject *val) {
 }
 
 static PyObject *_PylibMC_Pickle(PyObject *val) {
-    return PyObject_CallFunctionObjArgs(_PylibMC_pickle_dumps, val, NULL);
+    return PyObject_CallFunction(_PylibMC_pickle_dumps, "Oi", val, -1);
 }
 /* }}} */
 
@@ -2660,7 +2642,7 @@ static void _make_excs(PyObject *module) {
             "pylibmc.Error", NULL, NULL);
 
     PylibMCExc_CacheMiss = PyErr_NewException(
-            "_pylibmc.CacheMiss", PylibMCExc_Error, NULL);
+            "pylibmc.CacheMiss", PylibMCExc_Error, NULL);
 
     exc_objs = PyList_New(0);
     PyList_Append(exc_objs,
@@ -2670,7 +2652,7 @@ static void _make_excs(PyObject *module) {
 
     for (err = PylibMCExc_mc_errs; err->name != NULL; err++) {
         char excnam[64];
-        snprintf(excnam, 64, "_pylibmc.%s", err->name);
+        snprintf(excnam, 64, "pylibmc.%s", err->name);
         err->exc = PyErr_NewException(excnam, PylibMCExc_Error, NULL);
         PyObject_SetAttrString(err->exc, "retcode", PyLong_FromLong(err->rc));
         PyModule_AddObject(module, err->name, (PyObject *)err->exc);
